@@ -6,13 +6,13 @@ import re
 
 from copy import copy
 from types import FrameType, FunctionType, ModuleType
-from typing import Any, List, Optional, Tuple, Callable
+from typing import Any, Dict, List, Optional, Tuple, Callable
 
 from .complexity import is_complexity_tracing_enabled
-from .memory_footprint import MemoryFootprint
+from .memory_footprint import Event, MemoryFootprint
 from .utils import is_ipython_frame
 
-from ..utils import make_secret, pickle_and_hash, UnpicklableError
+from ..utils import make_secret, pickle_and_hash, UnpickleableError
 
 
 ACTIVE_FOOTPRINT = None
@@ -21,8 +21,8 @@ TRACING_VARNAME = "__PYBRYT_TRACING__"
 
 
 def create_collector(
-    skip_types: List[type] = [type, type(len), ModuleType, FunctionType], 
-    addl_filenames: List[str] = []
+    skip_types: List[type] = [type, type(len), FunctionType],
+    addl_filenames: List[str] = [],
 ) -> Tuple[MemoryFootprint, Callable[[FrameType, str, Any], Callable]]:
     """
     Creates a memory footprint to collect observed values and a trace function.
@@ -42,10 +42,10 @@ def create_collector(
         and the trace function
     """
     global ACTIVE_FOOTPRINT
-    vars_not_found = {}
+    vars_not_found: Dict[str, List[Tuple[str, str, int]]] = {}
     footprint = MemoryFootprint()
 
-    def track_value(val, seen_at=None):
+    def track_value(val: Any, event_name: str, seen_at: Optional[int] = None):
         """
         Tracks a value in ``footprint``. Checks that the value has not already been tracked by 
         pickling it and hashing the pickled object and comparing it to ``hashes``. If pickling is
@@ -53,13 +53,22 @@ def create_collector(
 
         Args:
             val (``object``): the object to be tracked
+            event_name (``str``): the event name provided by ``sys.settrace``
             seen_at (``int``, optional): an overriding step counter value
         """
         try:
+            if hasattr(val, "__module__"):
+                footprint.add_imports(val.__module__.split(".")[0])
+
             if type(val) in skip_types:
                 return
 
-            footprint.add_value(copy(val), seen_at)
+            if isinstance(val, ModuleType):
+                footprint.add_imports(val.__name__.split(".")[0])
+                return
+
+            event = Event.from_event_name(event_name)
+            footprint.add_value(copy(val), seen_at, event)
 
         # if something fails, don't track
         except:
@@ -111,43 +120,43 @@ def create_collector(
 
                         try:
                             val = eval(t, frame.f_globals, frame.f_locals)
-                            track_value(val)
+                            track_value(val, event)
                         except:
                             pass
 
                     else:
                         if t in frame.f_locals:
                             val = frame.f_locals[t]
-                            track_value(val)
+                            track_value(val, event)
                                 
                         elif t in frame.f_globals:
                             val = frame.f_globals[t]
-                            track_value(val)
+                            track_value(val, event)
                 
                 # for tracking the results of an assignment statement
                 m = re.match(r"^\s*(\w+)(\[[^\]]\]|(\.\w+)+)*\s=.*", line)
                 if m:
                     if name not in vars_not_found:
                         vars_not_found[name] = []
-                    vars_not_found[name].append((m.group(1), footprint.counter.get_value()))
+                    vars_not_found[name].append((m.group(1), event, footprint.counter.get_value()))
 
             if event == "return":
-                track_value(arg)
+                track_value(arg, event)
 
         elif (is_ipython_frame(frame) or frame.f_back.f_code.co_filename in addl_filenames) and \
                 event == "return":
-            track_value(arg)
+            track_value(arg, event)
 
         if event == "return" and name in vars_not_found:
             varnames = vars_not_found.pop(name)
-            for t, step in varnames:
+            for t, event_name, step in varnames:
                 if t in frame.f_locals:
                     val = frame.f_locals[t]
-                    track_value(val, step)
+                    track_value(val, event_name, step)
 
                 elif t in frame.f_globals:
                     val = frame.f_globals[t]
-                    track_value(val, step)
+                    track_value(val, event_name, step)
 
         return collect_intermidiate_results
 
@@ -259,6 +268,67 @@ class no_tracing:
         return False
 
 
+class FrameTracer:
+    """
+    A class for managing the tracing of a call stack.
+
+    Args:
+        frame (``FrameType``): the frame to initialize tracing in
+    """
+
+    footprint: Optional[MemoryFootprint]
+    """the memory footprint being populated"""
+
+    frame: FrameType
+    """the frame being traced"""
+
+    _tracing_already_enabled: bool
+    """whether tracing was already enabled when ``start_trace`` was called"""
+
+    def __init__(self, frame: FrameType) -> None:
+        self.frame = frame
+        self.footprint = None
+        self._tracing_already_enabled = False
+
+    def start_trace(self, **kwargs) -> None:
+        """
+        Create a collector and memory footprint and start tracing execution in the frame. Returns
+        a boolean indicating whether tracing was enabled.
+
+        Args:
+            **kwargs: additional keyword arguments passed to ``create_collector``
+
+        Returns:
+            ``bool``: whether this call initiated tracing (``False`` if tracing was already enabled)
+        """
+        self._tracing_already_enabled = get_tracing_frame() is not None
+        if self._tracing_already_enabled:
+            self.footprint = get_active_footprint()
+            return False
+
+        self.footprint, cir = create_collector(**kwargs)
+        self.frame.f_globals[TRACING_VARNAME] = True
+        tracing_on(tracing_func=cir)
+        return True
+
+    def end_trace(self) -> None:
+        """
+        End execution tracing in the frame.
+        """
+        if not self._tracing_already_enabled:
+            tracing_off(save_func=False)
+            self.frame.f_globals[TRACING_VARNAME] = False
+
+    def get_footprint(self) -> MemoryFootprint:
+        """
+        Return the memory footprint that was populated by the trace function.
+
+        Returns:
+            :py:class:`pybryt.execution.memory_footprint.MemoryFootprint`: the memory footprint
+        """
+        return self.footprint
+
+
 def get_active_footprint() -> Optional[MemoryFootprint]:
     """
     Get the active memory footprint if present, else ``None``.
